/* libfoXdesktop - foXdesktop's core library extending FOX
 * Copyright (C) 2000-2001 Josua Groeger <fractalus@gmx.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "FDApp.h"
#include "fd_defaults.h"
#include <iostream.h>

FXIMPLEMENT (FDApp, FXApp, NULL, 0);

FDApp::FDApp (const FXString &name= "Application", const FXString &vendor= "Default"): FXApp (name, vendor)
{
  iconDict= new FXIconDict (this);
}

FDApp::~FDApp ()
{
  delete iconDict;
}

void FDApp::init (int &argc, char **argv)
{
  FXApp::init (argc, argv);
  iconDict->setIconPath (reg ().readStringEntry ("SETTINGS", "iconpath", fd_default_iconpath));
}

void FDApp::showAbout (FXComposite *composite, const FXchar *name, const FXchar *version,
                       const FXchar *description, const FXchar *icon,
                       const FXchar *copyright, const FXchar *homepage)
{
  FXString caption (FXString ("About ")+ name);
  FXString body (name+ FXString (" ")+ version+ FXString (" - ")+ description+ FXString ("\n\n")+
                 copyright+ FXString ("\n")+ homepage+ FXString ("\n\n")+ name+
                 FXString (" depends on respectively\nbelongs to the foXdesktop project."
                 "\nSee http://foxdesktop.sourceforge.net for details."));
  
  FXMessageBox about (composite, caption, body, ico().insert (icon), MBOX_OK|DECOR_TITLE|DECOR_BORDER);
  about.execute ();
}

void FDApp::showHelp (const FXchar *app, const FXchar *file= "index.html")
{
  // Set lsStr e.g. to "ls -1 /foo/HTML/en/foXman/index.html /foo/bar/HTML/en/foXman/index.html"
  // (if docpath is "/foo:/foo/bar" and app is "foXman" and file is "index.html")
  FXString tmpStr, lsStr ("ls -1 ");
  lsStr.append (reg ().readStringEntry ("SETTINGS", "docpath", fd_default_docpath));
  lsStr+= ":";
  tmpStr= "/HTML/en/"+ FXString (app)+ "/"+ FXString (file)+ " ";
  for (int i= 0; lsStr[i]; i++) if (lsStr[i]== ':') lsStr.replace (i, 1, tmpStr);
  
  // Try to open lsStr
  FILE *pfile= popen (lsStr.text (), "r"); 
  if (!pfile) perror ("popen");
  else
  {
    char fileName[1024];
    if (!fgets (fileName, 1024, pfile))
      cerr<< "Could not find HTML help file "<< file<< " in section "<< app<< "!\n";
    else
    {
      // Cut spaces...
      char *ws= strpbrk (fileName, " \t\n");
      if (ws) *ws= 0;
        
      tmpStr= "";
      tmpStr.append (reg ().readStringEntry ("PROGRAM ALIASES", "xbrowser", fd_default_xbrowser));
      tmpStr+= " "; tmpStr.append (fileName); tmpStr+= " &";
      
      system (tmpStr.text ());
    }
  }
}
