/* libfoXdesktop - foXdesktop's core library extending FOX
 * Copyright (C) 2000-2001 Josua Groeger <fractalus@gmx.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "FDComponent.h"
#include "FDApp.h"

FXDEFMAP (FDComponent) FDComponentMap[]=
{
  FXMAPFUNC (SEL_COMMAND, FXWindow::ID_SHOW         , FDComponent::onCmdShow),
  
  FXMAPFUNC (SEL_COMMAND, FDComponent::ID_ABOUT     , FDComponent::onAbout),
  FXMAPFUNC (SEL_COMMAND, FDComponent::ID_HELP      , FDComponent::onHelp),

  FXMAPFUNC (SEL_COMMAND, FDComponent::ID_DEFAULTS  , FDComponent::onDefaults),
  FXMAPFUNC (SEL_COMMAND, FDComponent::ID_RELOAD    , FDComponent::onReload),
  
  FXMAPFUNC (SEL_COMMAND, FDComponent::ID_OK        , FDComponent::onOK),
  FXMAPFUNC (SEL_COMMAND, FDComponent::ID_APPLY     , FDComponent::onApply),
  FXMAPFUNC (SEL_COMMAND, FDComponent::ID_CANCEL    , FDComponent::onCancel),
};

FDCIMPLEMENT_ABSTRACT (FDComponent, FXPacker, FDComponentMap, ARRAYNUMBER (FDComponentMap));

FDComponent::FDComponent (const FXchar *type, const FXchar *component, FDCARGS): FXPacker (p, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
  // Registry
  FXString vkey ("foXdesktop-components_"); vkey.append (type);
  registry= new FXRegistry (component, vkey.text ());
  registry->read ();
  
  // Signals/Slots
  applySel= apply_sel;
  applyTarget= apply_target;
  applyPtr= apply_ptr;

  hideSel= hide_sel;
  hideTarget= hide_target;
  hidePtr= hide_ptr;
 
  // captionFrame
  FXHorizontalFrame *captionFrame= new FXHorizontalFrame (this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X,
                                                          0,0,0,0, 0,0,0,0, 0,0);
  new FXToolbarTab (captionFrame);
  FXToolbar *toolbar= new FXToolbar (captionFrame, LAYOUT_FILL_X|FRAME_RAISED);
   
  nameLabel= new FXLabel (toolbar, registry->readStringEntry ("SETTINGS", "caption", ""),
                          ((FDApp*) getApp ())->ico().insert (registry->readStringEntry
                          ("SETTINGS", "miniicon", "")), LAYOUT_CENTER_Y|ICON_BEFORE_TEXT|FRAME_THICK);
  FXButton *aboutButton= new FXButton (toolbar, "A&bout", ((FDApp*) getApp ())->ico().insert
                                   (registry->readStringEntry ("SETTINGS", "miniicon", "")), this, ID_ABOUT,
                                   LAYOUT_RIGHT|LAYOUT_CENTER_Y|BUTTON_TOOLBAR|FRAME_RAISED|ICON_BEFORE_TEXT);
  FXButton *helpButton= new FXButton (toolbar, "&Help", ((FDApp*) getApp ())->ico().insert
                                   ("mini/foXhelp.xpm"), this, ID_HELP,
                                   LAYOUT_RIGHT|LAYOUT_CENTER_Y|BUTTON_TOOLBAR|FRAME_RAISED|ICON_BEFORE_TEXT);
  // buttonFrame
  FXHorizontalFrame *buttonFrame= new FXHorizontalFrame (this, LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,
                                      0,0,0,0, 0,0,0,0, 0,0);
  new FXToolbarTab (buttonFrame);
  FXToolbar *buttonbar= new FXToolbar (buttonFrame, LAYOUT_FILL_X|FRAME_RAISED);

  FXButton *defaultButton= new FXButton (buttonbar, "&Default", NULL, this, ID_DEFAULTS, FRAME_RAISED);
  FXButton *reloadButton = new FXButton (buttonbar, "&Reload" , NULL, this, ID_RELOAD  , FRAME_RAISED);

  FXButton *cancelButton = new FXButton (buttonbar, "&Cancel",NULL, this,ID_CANCEL, LAYOUT_RIGHT|FRAME_RAISED);
  FXButton *applyButton  = new FXButton (buttonbar, "&Apply", NULL, this, ID_APPLY, LAYOUT_RIGHT|FRAME_RAISED);
  FXButton *okButton     = new FXButton (buttonbar, "&OK",    NULL, this, ID_OK   , LAYOUT_RIGHT|FRAME_RAISED);

  // Left/right spaces
  new FXPacker (this, LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y,  0,0,0,0, 0,0,0,0, 0,0);
  new FXPacker (this, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0, 0,0);

  // Disable buttons
  if (!registry->readIntEntry ("SETTINGS", "okbutton"     , 1)) okButton->hide ();
  if (!registry->readIntEntry ("SETTINGS", "applybutton"  , 0)) applyButton->hide ();
  if (!registry->readIntEntry ("SETTINGS", "cancelbutton" , 0)) cancelButton->hide ();
  
  if (!registry->readIntEntry ("SETTINGS", "reloadbutton" , 0)) reloadButton->hide ();
  if (!registry->readIntEntry ("SETTINGS", "defaultbutton", 0)) defaultButton->hide ();
  
  if (!registry->readIntEntry ("SETTINGS", "aboutbutton"  , 0)) aboutButton->disable ();
  if (!registry->readIntEntry ("SETTINGS", "helpbutton"   , 0)) helpButton->disable ();
}

FDComponent::~FDComponent ()
{
  delete registry;
}

void FDComponent::create ()
{
  FXPacker::create ();
  read ();
}

long FDComponent::onCmdShow (FXObject* obj, FXSelector sel, void *ptr)
{
  read ();
  return FXPacker::onCmdShow (obj, sel, ptr);
}

long FDComponent::onAbout (FXObject*, FXSelector, void*)
{
  about ();
}

long FDComponent::onHelp (FXObject*, FXSelector, void*)
{
  help ();
}

long FDComponent::onDefaults (FXObject*, FXSelector, void*)
{
  defaults ();
}

long FDComponent::onReload (FXObject*, FXSelector, void*)
{
  read ();
}

long FDComponent::onOK (FXObject*, FXSelector, void*)
{
  write ();
  if (applyTarget) applyTarget->handle (this, applySel, applyPtr);
  if (hideTarget) hideTarget->handle (this, hideSel, hidePtr);
}

long FDComponent::onApply (FXObject*, FXSelector, void*)
{
  write ();
  if (applyTarget) applyTarget->handle (this, applySel, applyPtr);
}

long FDComponent::onCancel (FXObject*, FXSelector, void*)
{
  if (hideTarget) hideTarget->handle (this, hideSel, hidePtr); 
}
