/* libfoXdesktop - foXdesktop's core library extending FOX
 * Copyright (C) 2000-2001 Josua Groeger <fractalus@gmx.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* FDComponent is the base class for all foXdesktop components,
 * i.e. if you want to make a foXdesktop component you have to derive it
 * from FDComponent. Furthermore you must use FDCDECLARE and FDCIMPLEMENT
 * instead of FXDECLARE and FXIMPLEMENT.
*/

#ifndef FDCOMPONENT_H
#define FDCOMPONENT_H

#include <fox/fx.h>

// The first version of libfoXdesktop with the current interface
#define FDC_INTERFACE_MAJOR  0
#define FDC_INTERFACE_MINOR  2
#define FDC_INTERFACE_SUBMIN 0

#define FDC_INTERFACE FDC_INTERFACE_MAJOR*1000000+ \
                      FDC_INTERFACE_MINOR*1000+ \
                      FDC_INTERFACE_SUBMIN

// Constructor arguments
#define FDCARGS \
  FXComposite *p, FXuint opts= 0, \
  FXObject *apply_target= NULL, FXSelector apply_sel= 0, void *apply_ptr= NULL, \
  FXObject *hide_target = NULL, FXSelector hide_sel = 0, void *hide_ptr = NULL, \
  FXint x= 0, FXint y= 0, FXint w= 0, FXint h= 0, \
  FXint pl= DEFAULT_SPACING, FXint pr= DEFAULT_SPACING, \
  FXint pt= DEFAULT_SPACING, FXint pb= DEFAULT_SPACING, \
  FXint hs= DEFAULT_SPACING, FXint vs= DEFAULT_SPACING

#define FDCPARAMS \
  p, opts, \
  apply_target, apply_sel, apply_ptr, \
  hide_target, hide_sel, hide_ptr, \
  x, y, w, h, pl, pr, pt, pb, hs, vs

// FXDECLARE... replacements for FDC-classes
#define FDCDECLARE(classname) \
  FXDECLARE(classname)

#define FDCIMPLEMENT(classname,baseclassname,mapping,nmappings) \
  extern "C" \
  { \
    extern const int classname##_interface= FDC_INTERFACE; \
    FDComponent *classname##_construct (FDCARGS) \
    { \
      return new classname (FDCPARAMS); \
    } \
  } \
  FXIMPLEMENT(classname,baseclassname,mapping,nmappings)

#define FDCDECLARE_ABSTRACT(classname) \
  FXDECLARE_ABSTRACT(classname)

#define FDCIMPLEMENT_ABSTRACT(classname,baseclassname,mapping,nmappings) \
  FXIMPLEMENT_ABSTRACT(classname,baseclassname,mapping,nmappings)

class FDComponent: public FXPacker
{
  FDCDECLARE_ABSTRACT (FDComponent)
    
  private:
    FXRegistry *registry;
  public:
    FXRegistry &reg () {return *registry;};
  
  protected:
    FDComponent () {};
    FDComponent (const FXchar *type, const FXchar *component, FDCARGS);
    
  public:
    virtual ~FDComponent ();
    virtual void create ();
    
    virtual void read       () {}; // Load the settings (read them and change the buttons, textedits... according to them)
    virtual void write      () {}; // Apply the settings (write them to the registry/configuration file...)
    virtual void defaults    () {}; // Change the buttons, textedits... so that they show the default settings
    
    virtual void setData (void *ptr) {};
    virtual void *getData () {return NULL;};
  
    virtual void about          () {};
    virtual void help           () {};
    
  protected:
    // Invoked by ok and apply
    FXObject *applyTarget;
    FXSelector applySel;
    void *applyPtr;

    // Invoked by ok and cancel
    FXObject *hideTarget;
    FXSelector hideSel;
    void *hidePtr;

    FXLabel *nameLabel;

  public:
    enum
    {
      ID_ABOUT= FXMainWindow::ID_LAST,
      ID_HELP,
      ID_DEFAULTS,
      ID_RELOAD,
      ID_OK,
      ID_APPLY,
      ID_CANCEL,
      ID_LAST
    };

    // public slots
    long onCmdShow  (FXObject*, FXSelector, void*);

    long onAbout    (FXObject*, FXSelector, void*);
    long onHelp     (FXObject*, FXSelector, void*);

    long onDefaults (FXObject*, FXSelector, void*);
    long onReload   (FXObject*, FXSelector, void*);

    long onOK       (FXObject*, FXSelector, void*);
    long onApply    (FXObject*, FXSelector, void*);
    long onCancel   (FXObject*, FXSelector, void*);
};

typedef FDComponent* TFDCConstruct (FDCARGS);

#endif
