/* libfoXdesktop - foXdesktop's core library extending FOX
 * Copyright (C) 2000-2001 Josua Groeger <fractalus@gmx.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* FDComponentConstructDict is a dict for managing foXdesktop components.
 * You can load multiple component libraries using insert (...) and
 * it maps the libraries' ltdl-handles to strings (format: "type/component",
 * e.g. "foXcontrol/DesktopColours") but this implementation is completely hidden
 * under the API. The internally used key string is split into its parts and
 * instead of the handles the find/remove... methods return a pointer
 * to the component's construction function (see FDComponent.h for details). */

#ifndef FDCOMPONENTCONSTRUCTDICT_H
#define FDCOMPONENTCONSTRUCTDICT_H

#include <ltdl.h>
#include "FDApp.h"
#include "FDComponent.h"

class FDComponentConstructDict: public FXDict
{
  FXDECLARE (FDComponentConstructDict)
  
  protected: 
    FDApp *app;
    FDComponentConstructDict () {};

    virtual void *createData (const void *ptr);
    virtual void deleteData (void *ptr);
  
  public:
    FDComponentConstructDict (FDApp *a);
    virtual ~FDComponentConstructDict ();
    
    // Insert one particular component (if available) respectively all components of one type
    TFDCConstruct *insert  (const FXchar *type, const FXchar *component, FXbool mrk= false);
    TFDCConstruct *insert  (const FXchar *type, FXbool mrk= false);
  
    TFDCConstruct *remove  (const FXchar *type, const FXchar *component);
    TFDCConstruct *find    (const FXchar *type, const FXchar *component);
    TFDCConstruct *data    (FXuint pos);
};

#endif
